package cz.muni.fi.pa165.entity;

import lombok.Getter;
import lombok.Setter;

import javax.persistence.*;

/**
 * @author mvalko
 */

@Setter
@Getter
@Entity
public class TeamCoachEntity extends PersonEntity {

    @ManyToOne
    @JoinColumn(name = "team_id_coach")
    private TeamEntity team;

    public TeamCoachEntity() {
    }

    public TeamCoachEntity(String name){
        super(name);
    }

    /**
     * Signs team for the tournament. It is to be considered as an application to the tournament. Does not guarantee that team is automatically part
     * of tournament.
     * @param tournament TournamentEntity which team wants to attend.
     */
    public void signForTournament(TournamentEntity tournament) {
        tournament.signTeamForTournament(this.team);
    }

    /**
     * Adds player who recently joined the team to the team.
     * @param newPlayer PlayerEntity that was the most recent addition to the team.
     */
    public void addPlayerToTheTeam(PlayerEntity newPlayer){ this.team.addPlayerToTheTeam(newPlayer);}

    /**
     * Removes player who recently left the team.
     * @param oldPlayer PlayerEntity that was the most recent fired from the team.
     */
    public void removePlayerFromTheTeam(PlayerEntity oldPlayer){ this.team.removePlayerFromTheTeam(oldPlayer);}

    /**
     * Adds coach who recently joined the team to the team.
     * @param newCoach TeamCoachEntity that was the most recent addition to the team.
     */
    public void addCoachToTheTeam(TeamCoachEntity newCoach){
        this.team.addCoachToTheTeam(newCoach);
        newCoach.setTeam(this.team);
    }

    /**
     * Removes coach who recently left the team.
     * @param oldCoach TeamCoachEntity that was the most recent fired from the team.
     */
    public void removeCoachFromTheTeam(TeamCoachEntity oldCoach){
        this.team.removeCoachFromTheTeam(oldCoach);
        oldCoach.setTeam(null);
    }
}
