package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashSet;
import java.util.List;

/**
 * @author Adrian Piacek
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class MatchEntityTest {
    private final EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");

    private MatchEntity matchA, matchB, matchC;
    private TeamEntity teamA,teamB,teamC,teamD,teamE,teamF;
    private PlayerEntity playerA,playerB,playerC,playerD,playerE,playerF;
    private TournamentEntity tournamentA;

    @BeforeAll
    void init(){
        matchA = new MatchEntity();
        matchB = new MatchEntity();
        matchC = new MatchEntity();

        teamA = new TeamEntity();
        teamB = new TeamEntity();
        teamC = new TeamEntity();
        teamD = new TeamEntity();
        teamE = new TeamEntity();
        teamF = new TeamEntity();
        teamA.setName("A");
        teamB.setName("B");
        teamC.setName("C");
        teamD.setName("D");
        teamE.setName("E");
        teamF.setName("F");

        playerA = new PlayerEntity("A");
        playerB = new PlayerEntity("B");
        playerC = new PlayerEntity("C");
        playerD = new PlayerEntity("D");
        playerE = new PlayerEntity("E");
        playerF = new PlayerEntity("F");

        matchA.setTeams(new HashSet<>(List.of(teamA, teamB)));
        matchB.setTeams(new HashSet<>(List.of(teamC, teamD, teamE)));
        matchC.setTeams(new HashSet<>(List.of(teamA, teamC, teamF)));

        teamA.setMatches(new HashSet<>(List.of(matchA, matchC)));
        teamB.setMatches(new HashSet<>(List.of(matchA)));
        teamC.setMatches(new HashSet<>(List.of(matchB,matchC)));
        teamD.setMatches(new HashSet<>(List.of(matchB)));
        teamE.setMatches(new HashSet<>(List.of(matchB)));
        teamF.setMatches(new HashSet<>(List.of(matchC)));

        tournamentA = new TournamentEntity("MSI","Osaka", CountryEnum.SLOVAKIA, 1000, TournamentFormat.DEFAULT, 10);
        matchA.setTournament(tournamentA);
        matchB.setTournament(tournamentA);
        tournamentA.setMatches(new HashSet<>(List.of(matchA, matchB)));

        matchA.setPlayers(new HashSet<>(List.of(playerA,playerB,playerC,playerD,playerE,playerF)));
        matchB.setPlayers(new HashSet<>(List.of(playerA,playerB,playerC,playerD)));
        matchC.setPlayers(new HashSet<>(List.of(playerB,playerD,playerE,playerF)));
        playerA.setMatches(new HashSet<>(List.of(matchA, matchB)));
        playerB.setMatches(new HashSet<>(List.of(matchA, matchB, matchC)));
        playerC.setMatches(new HashSet<>(List.of(matchA, matchB)));
        playerD.setMatches(new HashSet<>(List.of(matchA, matchB, matchC)));
        playerE.setMatches(new HashSet<>(List.of(matchA, matchC)));
        playerF.setMatches(new HashSet<>(List.of(matchA, matchC)));

        EntityManager em = emf.createEntityManager();
        try{
            em.getTransaction().begin();
            em.persist(matchA);
            em.persist(matchB);
            em.persist(matchC);

            em.persist(teamA);
            em.persist(teamB);
            em.persist(teamC);
            em.persist(teamD);
            em.persist(teamE);
            em.persist(teamF);

            em.persist(playerA);
            em.persist(playerB);
            em.persist(playerC);
            em.persist(playerD);
            em.persist(playerE);
            em.persist(playerF);

            em.persist(tournamentA);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void attributesTests(){
        MatchEntity match;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            match = em1.find(MatchEntity.class, matchA.getId());
            Assertions.assertEquals(match.getId(), matchA.getId());
            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
    }

    /**
     * Testing relationships between TournamentEntity and MatchEntity
     * Tests whether match assignment to tournament corresponds to the correct tournament.
     * Tests whether tournament consists of corresponding matches.
     */
    @Test
    void relationshipTestTournament(){
        MatchEntity match;
        TournamentEntity tournament;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            match = em1.find(MatchEntity.class, matchA.getId());
            Assertions.assertEquals(match.getTournament(), tournamentA);

            tournament = em1.find(TournamentEntity.class, tournamentA.getId());
            Assertions.assertEquals(tournament.getMatches(), new HashSet<>(List.of(matchA, matchB)));
            match = em1.find(MatchEntity.class, matchB.getId());
            Assertions.assertEquals(match.getTournament(), tournamentA);

            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
    }

    @Test
    void relationshipTestTeam(){
        MatchEntity match;
        TeamEntity team;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            match = em1.find(MatchEntity.class, matchA.getId());
            Assertions.assertEquals(match.getTeams(), new HashSet<>(List.of(teamA,teamB)));

            match = em1.find(MatchEntity.class, matchB.getId());
            Assertions.assertEquals(match.getTeams(), new HashSet<>(List.of(teamC, teamD, teamE)));

            match = em1.find(MatchEntity.class, matchC.getId());
            Assertions.assertEquals(match.getTeams(), new HashSet<>(List.of(teamA, teamC, teamF)));

            team = em1.find(TeamEntity.class, teamB.getId());
            Assertions.assertEquals(team.getMatches(), new HashSet<>(List.of(matchA)));

            team = em1.find(TeamEntity.class, teamA.getId());
            Assertions.assertEquals(team.getMatches(), new HashSet<>(List.of(matchA, matchC)));

            team = em1.find(TeamEntity.class, teamC.getId());
            Assertions.assertEquals(team.getMatches(), new HashSet<>(List.of(matchB,matchC)));
            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
    }

    /**
     * Testing relationships between PlayerEntity and MatchEntity
     * Tests whether match assignment of Players is corresponding to the default list of players for the tournament.
     * Tests whether player is assigned to the corresponding match as planned.
     */
    @Test
    void relationshipTestPlayers(){
        MatchEntity match;
        PlayerEntity player;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            match = em1.find(MatchEntity.class, matchA.getId());
            Assertions.assertEquals(match.getPlayers(),
                    new HashSet<>(List.of(playerA,playerB,playerC,playerD,playerE,playerF)));

            match = em1.find(MatchEntity.class, matchB.getId());
            Assertions.assertEquals(match.getPlayers(),
                    new HashSet<>(List.of(playerA,playerB,playerC,playerD)));

            match = em1.find(MatchEntity.class, matchC.getId());
            Assertions.assertEquals(match.getPlayers(),
                    new HashSet<>(List.of(playerB,playerD,playerE,playerF)));

            player = em1.find(PlayerEntity.class, playerA.getId());
            Assertions.assertEquals(player.getMatches(), new HashSet<>(List.of(matchA, matchB)));

            player = em1.find(PlayerEntity.class, playerB.getId());
            Assertions.assertEquals(player.getMatches(), new HashSet<>(List.of(matchA, matchB, matchC)));

            player = em1.find(PlayerEntity.class, playerD.getId());
            Assertions.assertEquals(player.getMatches(), new HashSet<>(List.of(matchA, matchB, matchC)));

            player = em1.find(PlayerEntity.class, playerF.getId());
            Assertions.assertEquals(player.getMatches(), new HashSet<>(List.of(matchA, matchC)));

            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
    }

    @AfterAll
    public void afterAll() {
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.createQuery("delete from MatchEntity").executeUpdate();
            em.createQuery("delete from TeamEntity").executeUpdate();
            em.createQuery("delete from TournamentEntity").executeUpdate();
            em.createQuery("delete from PlayerEntity").executeUpdate();
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }
}
