package cz.muni.fi.pa165.sampledata;

import com.google.common.hash.Hashing;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import cz.muni.fi.pa165.services.player.PlayerService;
import cz.muni.fi.pa165.services.team.TeamService;
import cz.muni.fi.pa165.services.tournament.TournamentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.argon2.Argon2PasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * @author Adrián Piaček
 */
@Component
@Transactional //transactions are handled on facade layer
public class SampleDataLoadingFacadeImpl implements SampleDataLoadingFacade {

    final static Logger log = LoggerFactory.getLogger(SampleDataLoadingFacadeImpl.class);

    @Autowired
    private PlayerService playerService;

    @Autowired
    private TeamService teamService;

    @Autowired
    private TournamentService tournamentService;

    @Override
    @SuppressWarnings("unused")
    public void loadData() throws IOException {
        TeamEntity g2_esports = team("g2_esports", CountryEnum.GERMANY);
        TeamEntity skt_t1 = team("skt_t1", CountryEnum.SOUTH_KOREA);

        PlayerEntity crimsix = player("9384b466-badf-40ca-bbeb-4d8f6eb98042", "Ian Porter", "poter",
                "heslo", true, CountryEnum.AFGHANISTAN, g2_esports, 5, 6, 7);

        PlayerEntity JJoNak = player("6e374a4e-5213-4581-847c-45f9e73ffe90", "Bang Seong-hyun ", "bang",
                "heslo", true, CountryEnum.ALBANIA, skt_t1, 15, 25, 100);
        log.info("Loaded Sample Players.");

        TournamentEntity msi = tournament("MSI", "Ottawa" ,CountryEnum.SOUTH_KOREA, 10000, TournamentFormat.DEFAULT, 10);
        TournamentEntity worlds = tournament("Worlds", "Zilina", CountryEnum.CHINA, 50000, TournamentFormat.DEFAULT, 6);
    }

    private PlayerEntity player(String uuid, String name, String username, String password, boolean admin,
                                CountryEnum country, TeamEntity team, int killStat, int deathStat, int assistStat) {
        PlayerEntity player = new PlayerEntity();
        player.setId(uuid);
        player.setName(name);
        player.setUsername(username);
        String sha256hex = Hashing.sha256().hashString(password, StandardCharsets.UTF_8).toString();
        player.setPasswordHash(sha256hex);
        player.setAdmin(admin);
        player.setCountry(country);
        player.setTeam(team);
        player.setKillStat(killStat);
        player.setDeathStat(deathStat);
        player.setAssistStat(assistStat);
        playerService.create(player);
        return player;
    }

    private TeamEntity team(String name, CountryEnum country) {
        TeamEntity team = new TeamEntity();
        team.setName(name);
        team.setCountry(country);
        teamService.create(team);
        return team;
    }

    private TournamentEntity tournament(String name, String city, CountryEnum country, double prizePool, TournamentFormat format,
                                        int teamCapacity) {
        TournamentEntity tournament = new TournamentEntity();
        tournament.setName(name);
        tournament.setCountry(country);
        tournament.setCity(city);
        tournament.setPrizePool(prizePool);
        tournament.setTournamentFormat(format);
        tournament.setTeamCapacity(teamCapacity);
        tournamentService.create(tournament);
        return tournament;
    }

}
