package cz.muni.fi.pa165.mvc.controllers;

import cz.muni.fi.pa165.dto.PersonAuthenticateDTO;
import cz.muni.fi.pa165.dto.PersonDTO;
import cz.muni.fi.pa165.entity.PersonEntity;
import cz.muni.fi.pa165.services.BeanMappingService;
import cz.muni.fi.pa165.services.person.PersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpSession;
import javax.validation.Valid;

/**
 * @author Marek Kadlečík
 */
@Controller
@RequestMapping("/security")
public class LoginController {

    @Autowired
    private PersonService personService;

    @Autowired
    private BeanMappingService mapper;


    @GetMapping(value = "/login")
    public String getLogin(Model model, HttpSession session) {
        if (session.getAttribute("authenticatedUser") != null) {
            return "redirect:/";
        }
        model.addAttribute("user", new PersonAuthenticateDTO());
        return "/security/login";
    }

    @PostMapping(value = "/login")
    public String postLogin(Model model, HttpSession session, @Valid @ModelAttribute("user") PersonDTO personDTO,
                            BindingResult bindingResult, RedirectAttributes redirectAttributes) {


        if (bindingResult.hasErrors()) {
            redirectAttributes.addFlashAttribute("alert_danger", "Login failed!");
            return "security/loginFailed";
        }
        PersonEntity person = personService.findByUsername(personDTO.getUsername());

        if (person == null) {
            redirectAttributes.addFlashAttribute("alert_danger", "Login failed!");
            return "security/loginFailed";
        }

        if (!personService.authenticate(person, personDTO.getPasswordHash())) {
            redirectAttributes.addFlashAttribute("alert_danger", "Login failed!");
            return "security/loginFailed";
        }

        session.setAttribute("authenticatedUser", mapper.mapTo(person, PersonDTO.class));
        redirectAttributes.addFlashAttribute("alert_success", "Login successful!");
        return "redirect:/";
    }

    @GetMapping(value = "/logout")
    public String logout(Model model, HttpSession session, RedirectAttributes redirectAttributes) {
        if (session.getAttribute("authenticatedUser") == null) {
            return "redirect:/";
        }
        session.removeAttribute("authenticatedUser");
        redirectAttributes.addFlashAttribute("alert_success", "Logout successful!");
        return "/security/login";
    }

    @GetMapping(value = "/wrongAccess")
    public String wrongAccess(Model model, HttpSession httpSession) {
        PersonDTO personDTO = (PersonDTO) httpSession.getAttribute("authenticatedUser");
        model.addAttribute("authenticatedUser", personDTO);
        return "/security/wrongAccess";
    }
}
