package cz.muni.fi.pa165.mvc.controllers;

import cz.muni.fi.pa165.dto.MatchDTO;
import cz.muni.fi.pa165.dto.TeamDTO;
import cz.muni.fi.pa165.dto.TournamentDTO;
import cz.muni.fi.pa165.facade.MatchFacade;
import cz.muni.fi.pa165.facade.TeamFacade;
import cz.muni.fi.pa165.facade.TournamentFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.util.UriComponentsBuilder;

import javax.validation.Valid;
import java.util.List;

/**
 * @author Dominik Kozubík
 */
@Controller
@RequestMapping(value = "/match")
public class MatchController {

    @Autowired
    private MatchFacade matchFacade;

    @Autowired
    private TournamentFacade tournamentFacade;

    @Autowired
    private TeamFacade teamFacade;

    @ModelAttribute("tournaments")
    public List<TournamentDTO> tournaments() {
        return tournamentFacade.findAllTournament();
    }

    @ModelAttribute("teams")
    public List<TeamDTO> teams() {
        return teamFacade.findAllTeams();
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public String list(Model model) {
        List<MatchDTO> matchDTOList = matchFacade.findAllMatches();
        model.addAttribute("matches", matchDTOList);
        return "/match/list";
    }

    @RequestMapping(value = "/new", method = RequestMethod.GET)
    public String newMatch(Model model) {
        model.addAttribute("matchCreate", new MatchDTO());
        return "match/new";
    }

    @RequestMapping(value = "/view/{uuid}", method = RequestMethod.GET)
    public String view(@PathVariable String uuid, Model model) {
        model.addAttribute("match", matchFacade.getMatchById(uuid));
        return "match/view";
    }

    @RequestMapping(value = "/delete/{uuid}", method = RequestMethod.POST)
    public String delete(@PathVariable String uuid, UriComponentsBuilder uriBuilder, RedirectAttributes redirectAttributes) {
        MatchDTO match = matchFacade.getMatchById(uuid);
        try {
            matchFacade.delete(uuid);
            redirectAttributes.addFlashAttribute("alert_success", "Match \"" + match.getId() + "\" was deleted.");
        } catch (Exception ex) {
            redirectAttributes.addFlashAttribute("alert_danger", "Match \"" + match.getId() + "\" cannot be deleted.");
        }
        return "redirect:" + uriBuilder.path("/match/list").toUriString();
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST)
    public String create(@Valid @ModelAttribute("matchCreate") MatchDTO formBean, BindingResult bindingResult,
                         Model model, RedirectAttributes redirectAttributes, UriComponentsBuilder uriBuilder) {
        //in case of validation error forward back to the form
        if (bindingResult.hasErrors()) {
            return "match/new";
        }
        //create match
        String uuid = matchFacade.createMatch(formBean);
        //report success
        redirectAttributes.addFlashAttribute("alert_success", "Match " + uuid + " was created");
        return "redirect:" + uriBuilder.path("/match/view/{id}").buildAndExpand(uuid).encode().toUriString();
    }
}
