package cz.muni.fi.pa165.mvc.controllers;

import cz.muni.fi.pa165.dto.TournamentDTO;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import cz.muni.fi.pa165.facade.TournamentFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.NestedExceptionUtils;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.util.UriComponentsBuilder;

import javax.validation.Valid;
import java.util.List;

/**
 * @author Marek Kadlečík
 */
@Controller
@RequestMapping("/tournament")
public class TournamentController {

    @Autowired
    private TournamentFacade tournamentFacade;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public String list(Model model) {
        List<TournamentDTO> tournaments = tournamentFacade.findAllTournament();
        model.addAttribute("tournaments", tournaments);
        return "tournament/list";
    }

    @RequestMapping(value = "/new", method = RequestMethod.GET)
    public String newProduct(Model model) {
        model.addAttribute("tournamentsCreate", new TournamentDTO());
        return "tournament/new";
    }

    @ModelAttribute("countries")
    public CountryEnum[] countries() {
        return CountryEnum.values();
    }

    @ModelAttribute("formats")
    public TournamentFormat[] formats() {
        return TournamentFormat.values();
    }

    @RequestMapping(value = "/view/{uuid}", method = RequestMethod.GET)
    public String view(@PathVariable String uuid, Model model) {
        model.addAttribute("tournament", tournamentFacade.getTournamentById(uuid));
        return "tournament/view";
    }

    @RequestMapping(value = "/delete/{uuid}", method = RequestMethod.POST)
    public String delete(@PathVariable String uuid, UriComponentsBuilder uriBuilder, RedirectAttributes redirectAttributes) {
        TournamentDTO tournament = tournamentFacade.getTournamentById(uuid);
        try {
            tournamentFacade.delete(uuid);
            redirectAttributes.addFlashAttribute("alert_success", "Tournament \"" + tournament.getName() + "\" was deleted.");
        } catch (Exception ex) {
            redirectAttributes.addFlashAttribute("alert_danger", "Tournament \"" + tournament.getName() + "\" cannot be deleted.");
        }
        return "redirect:" + uriBuilder.path("/tournament/list").toUriString();
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST)
    public String create(@Valid @ModelAttribute("tournamentCreate") TournamentDTO formBean, BindingResult bindingResult,
                         Model model, RedirectAttributes redirectAttributes, UriComponentsBuilder uriBuilder) {
        //in case of validation error forward back to the the form
        if (bindingResult.hasErrors()) {
            return "tournament/new";
        }
        //create tournament
        String uuid = tournamentFacade.createTournament(formBean);
        //report success
        redirectAttributes.addFlashAttribute("alert_success", "Tournament " + formBean.getName() + " was created");
        return "redirect:" + uriBuilder.path("/tournament/view/{id}").buildAndExpand(uuid).encode().toUriString();
    }
}
